@@ -13,7 +13,7 @@
 import mock
 import testtools
 
-from openstack import exceptions
+from openstack.exceptions import SDKException
 from openstack.tests.unit import base
 
 from openstack.network.v2 import router
@@ -121,7 +121,6 @@ class TestRouter(base.TestCase):
         response = mock.Mock()
         response.body = {"subnet_id": "3", "port_id": "2"}
         response.json = mock.Mock(return_value=response.body)
-        response.status_code = 200
         sess = mock.Mock()
         sess.put = mock.Mock(return_value=response)
         body = {"subnet_id": "3"}
@@ -137,7 +136,6 @@ class TestRouter(base.TestCase):
         response = mock.Mock()
         response.body = {"subnet_id": "3", "port_id": "3"}
         response.json = mock.Mock(return_value=response.body)
-        response.status_code = 200
         sess = mock.Mock()
         sess.put = mock.Mock(return_value=response)
 
@@ -154,7 +152,6 @@ class TestRouter(base.TestCase):
         response = mock.Mock()
         response.body = {"subnet_id": "3", "port_id": "2"}
         response.json = mock.Mock(return_value=response.body)
-        response.status_code = 200
         sess = mock.Mock()
         sess.put = mock.Mock(return_value=response)
         body = {"subnet_id": "3"}
@@ -170,7 +167,6 @@ class TestRouter(base.TestCase):
         response = mock.Mock()
         response.body = {"subnet_id": "3", "port_id": "3"}
         response.json = mock.Mock(return_value=response.body)
-        response.status_code = 200
         sess = mock.Mock()
         sess.put = mock.Mock(return_value=response)
         body = {"network_id": 3, "enable_snat": True}
@@ -184,16 +180,15 @@ class TestRouter(base.TestCase):
         # Neutron may return 4xx, we have to raise if that happens
         sot = router.Router(**EXAMPLE)
         response = mock.Mock()
-        msg = '.*borked'
+        msg = 'borked'
         response.body = {'NeutronError': {'message': msg}}
         response.json = mock.Mock(return_value=response.body)
         response.ok = False
         response.status_code = 409
-        response.headers = {'content-type': 'application/json'}
         sess = mock.Mock()
         sess.put = mock.Mock(return_value=response)
         body = {'subnet_id': '3'}
-        with testtools.ExpectedException(exceptions.ConflictException, msg):
+        with testtools.ExpectedException(SDKException, msg):
             sot.add_interface(sess, **body)
 
     def test_remove_interface_4xx(self):
@@ -201,18 +196,49 @@ class TestRouter(base.TestCase):
         # extra routes referring to it as a nexthop
         sot = router.Router(**EXAMPLE)
         response = mock.Mock()
-        msg = '.*borked'
+        msg = 'borked'
         response.body = {'NeutronError': {'message': msg}}
         response.json = mock.Mock(return_value=response.body)
         response.ok = False
         response.status_code = 409
-        response.headers = {'content-type': 'application/json'}
         sess = mock.Mock()
         sess.put = mock.Mock(return_value=response)
         body = {'subnet_id': '3'}
-        with testtools.ExpectedException(exceptions.ConflictException, msg):
+        with testtools.ExpectedException(SDKException, msg):
             sot.remove_interface(sess, **body)
 
+    def test_add_extra_routes(self):
+        r = router.Router(**EXAMPLE)
+        response = mock.Mock()
+        response.headers = {}
+        json_body = {'router': {}}
+        response.body = json_body
+        response.json = mock.Mock(return_value=response.body)
+        response.status_code = 200
+        sess = mock.Mock()
+        sess.put = mock.Mock(return_value=response)
+        ret = r.add_extra_routes(sess, json_body)
+        self.assertIsInstance(ret, router.Router)
+        self.assertIsInstance(ret.routes, list)
+        url = 'routers/IDENTIFIER/add_extraroutes'
+        sess.put.assert_called_with(url, json=json_body)
+
+    def test_remove_extra_routes(self):
+        r = router.Router(**EXAMPLE)
+        response = mock.Mock()
+        response.headers = {}
+        json_body = {'router': {}}
+        response.body = json_body
+        response.json = mock.Mock(return_value=response.body)
+        response.status_code = 200
+        sess = mock.Mock()
+        sess.put = mock.Mock(return_value=response)
+        ret = r.remove_extra_routes(sess, json_body)
+        self.assertIsInstance(ret, router.Router)
+        self.assertIsInstance(ret.routes, list)
+        url = 'routers/IDENTIFIER/remove_extraroutes'
+        sess.put.assert_called_with(url, json=json_body)
+
     def test_add_router_gateway(self):
         # Add gateway to a router
         sot = router.Router(**EXAMPLE_WITH_OPTIONAL)
